from rest_framework import serializers


from districts.models import *


# Regions Serializer Class
class Regions_Serializer(serializers.ModelSerializer):
    class Meta:
        model = Regions
        fields = '__all__'


# Districts Serializer Class
class Districts_Serializer(serializers.ModelSerializer):
    class Meta:
        model = Districts
        fields = '__all__'

# Districts ListSerializer Class
class Districts_ListSerializer(serializers.ModelSerializer):
    region = serializers.SlugRelatedField(read_only=True, slug_field="region")
    
    class Meta:
        model = Districts
        fields = '__all__'


# County Serializer Class
class County_Serializer(serializers.ModelSerializer):
    class Meta:
        model = County
        fields = '__all__'


# County ListSerializer Class
class County_ListSerializer(serializers.ModelSerializer):
    district = serializers.SlugRelatedField(read_only=True, slug_field="district")

    class Meta:
        model = County
        fields = '__all__'


# SubCounty Serializer Class
class SubCounty_Serializer(serializers.ModelSerializer):
    class Meta:
        model = SubCounty
        fields = '__all__'


# SubCounty ListSerializer Class
class SubCounty_ListSerializer(serializers.ModelSerializer):
    county = serializers.SlugRelatedField(read_only=True, slug_field="county")

    class Meta:
        model = SubCounty
        fields = '__all__'


# Parish Serializer Class
class Parish_Serializer(serializers.ModelSerializer):
    class Meta:
        model = Parish
        fields = '__all__'


# Parish ListSerializer Class
class Parish_ListSerializer(serializers.ModelSerializer):
    sub_county = serializers.SlugRelatedField(read_only=True, slug_field="sub_county")

    class Meta:
        model = Parish
        fields = '__all__'


# Location Serializer Class
class Location_Serializer(serializers.ModelSerializer):
    class Meta:
        model = Location
        fields = '__all__'



from django.core.management.base import BaseCommand
from django.core.management import call_command


class Command(BaseCommand):
    help = 'Loads the initial data for the district package, python manage.py load_districts_data'

    def handle(self, *args, **options):
        call_command('loaddata', 'data.json', app_label='districts')


from django.db import models

# Regions Model.
class Regions(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    region = models.CharField(max_length=20)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.region
    
    class Meta:
        verbose_name = "Region"
        verbose_name_plural = "Region"


# Districts/City Model.
class Districts(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    region = models.ForeignKey(Regions, on_delete=models.CASCADE)
    district = models.CharField(max_length=50)
    population = models.IntegerField()
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.district
    
    class Meta:
        verbose_name = "District/City"
        verbose_name_plural = "District/City"


# County/Municipality Model.
class County(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    district = models.ForeignKey(Districts, on_delete=models.CASCADE)
    county = models.CharField(max_length=50)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.county
    
    class Meta:
        verbose_name = "County/Municipality"
        verbose_name_plural = "County/Municipality"


# SubCounty Model.
class SubCounty(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    county = models.ForeignKey(County, on_delete=models.CASCADE)
    sub_county = models.CharField(max_length=50)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.sub_county
    
    class Meta:
        verbose_name = "Sub-County/Town Council/Division"
        verbose_name_plural = "Sub-County/Town Council/Division"


# Parish Model.
class Parish(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    sub_county = models.ForeignKey(SubCounty, on_delete=models.CASCADE)
    parish = models.CharField(max_length=50)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.parish
    
    class Meta:
        verbose_name = "Parish/Ward"
        verbose_name_plural = "Parish/Ward"


# Location Model.
class Location(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    latitude = models.FloatField()
    longitude = models.FloatField()
    parish = models.OneToOneField(Parish, on_delete=models.CASCADE)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"Lat: { self.latitude }, Long: { self.longitude }"
    
    class Meta:
        verbose_name = "Location"
        verbose_name_plural = "Location"



from django.urls import path, include
from rest_framework.routers import DefaultRouter

from districts.views import *

router = DefaultRouter()
router.register(r'region', Regions_ViewSet, basename='Regions_ViewSet')
router.register(r'district', Districts_ViewSet, basename='Districts_ViewSet')
router.register(r'county', County_ViewSet, basename='County_ViewSet')
router.register(r'subcounty', SubCounty_ViewSet, basename='SubCounty_ViewSet')
router.register(r'parish', Parish_ViewSet, basename='Parish_ViewSet')
router.register(r'location', Location_ViewSet, basename='Location_ViewSet')

urlpatterns = [
    path('', include(router.urls)),
]


from rest_framework import viewsets, status
from rest_framework.response import Response

from django.shortcuts import get_object_or_404

from districts.serializers import *


# Regions ViewSet Class
class Regions_ViewSet(viewsets.ViewSet):
    
    def list(self, request): # GET method for list of all Regions handled here
        queryset = Regions.objects.all()
        serializer = Regions_Serializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create(self, request): # POST method handled here
        serializer = Regions_Serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    def retrieve(self, request, pk=None): # GET method for a specific Region handled here
        try:
            queryset = get_object_or_404(Regions, pk=pk)
        except Regions.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = Regions_Serializer(queryset)
        return Response(serializer.data, status=status.HTTP_302_FOUND)

    def update(self, request, pk=None): # PUT method handled here - Updating record
        try:
            queryset = Regions.objects.get(pk=pk)
        except Regions.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = Regions_Serializer(queryset, data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def destroy(self, request, pk=None): # DELETE method handled here
        try:
            queryset = Regions.objects.get(pk=pk)
        except Regions.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        queryset.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


# Districts ViewSet Class
class Districts_ViewSet(viewsets.ViewSet):
    
    def list(self, request): # GET method for list of all Districts handled here
        queryset = Districts.objects.all()
        serializer = Districts_ListSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create(self, request): # POST method handled here
        if request.data.get("action"):
            region = request.data.get("action")
            if type(region) != int:
                region_obj = Regions.objects.get(region=region)
                region = region_obj.id
            queryset = Districts.objects.filter(region=region_obj.id)
            serializer = Districts_ListSerializer(queryset, many=True)
            return Response(serializer.data, status=status.HTTP_200_OK)
        serializer = Districts_Serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    def retrieve(self, request, pk=None): # GET method for a specific District handled here
        try:
            queryset = get_object_or_404(Districts, pk=pk)
        except Districts.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = Districts_Serializer(queryset)
        return Response(serializer.data, status=status.HTTP_302_FOUND)


# County ViewSet Class
class County_ViewSet(viewsets.ViewSet):
    
    def list(self, request): # GET method for list of all County handled here
        queryset = County.objects.all()
        serializer = County_ListSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create(self, request): # POST method handled here
        if request.data.get("action"):
            district = request.data.get("action")
            if type(district) != int:
                district_obj = Districts.objects.get(district=district)
                district = district_obj.id
            queryset = County.objects.filter(district=district)
            serializer = County_ListSerializer(queryset, many=True)
            return Response(serializer.data, status=status.HTTP_200_OK)
        serializer = County_Serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    def retrieve(self, request, pk=None): # GET method for a specific County handled here
        try:
            queryset = get_object_or_404(County, pk=pk)
        except County.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = County_Serializer(queryset)
        return Response(serializer.data, status=status.HTTP_302_FOUND)


# SubCounty ViewSet Class
class SubCounty_ViewSet(viewsets.ViewSet):
    
    def list(self, request): # GET method for list of all SubCounty handled here
        queryset = SubCounty.objects.all()
        serializer = SubCounty_ListSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create(self, request): # POST method handled here
        if request.data.get("action"):
            county = request.data.get("action")
            if type(county) != int:
                county_obj = County.objects.get(county=county)
                county = county_obj.id
            queryset = SubCounty.objects.filter(county=county)
            serializer = SubCounty_ListSerializer(queryset, many=True)
            return Response(serializer.data, status=status.HTTP_200_OK)
        serializer = SubCounty_Serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    def retrieve(self, request, pk=None): # GET method for a specific SubCounty handled here
        try:
            queryset = get_object_or_404(SubCounty, pk=pk)
        except SubCounty.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = SubCounty_Serializer(queryset)
        return Response(serializer.data, status=status.HTTP_302_FOUND)


# Parish ViewSet Class
class Parish_ViewSet(viewsets.ViewSet):
    
    def list(self, request): # GET method for list of all Parish handled here
        queryset = Parish.objects.all()
        serializer = Parish_ListSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create(self, request): # POST method handled here
        if request.data.get("action"):
            sub_county = request.data.get("action")
            if type(sub_county) != int:
                sub_county_obj = SubCounty.objects.get(sub_county=sub_county)
                sub_county = sub_county_obj.id
            queryset = Parish.objects.filter(sub_county=sub_county)
            serializer = Parish_ListSerializer(queryset, many=True)
            return Response(serializer.data, status=status.HTTP_200_OK)
        serializer = Parish_Serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    def retrieve(self, request, pk=None): # GET method for a specific Parish handled here
        try:
            queryset = get_object_or_404(Parish, pk=pk)
        except Parish.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = Parish_Serializer(queryset)
        return Response(serializer.data, status=status.HTTP_302_FOUND)


# Location ViewSet Class
class Location_ViewSet(viewsets.ViewSet):
    
    def list(self, request): # GET method for list of all Location handled here
        queryset = Location.objects.all()
        serializer = Location_Serializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create(self, request): # POST method handled here
        serializer = Location_Serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    def retrieve(self, request, pk=None): # GET method for a specific District handled here
        try:
            queryset = get_object_or_404(Location, pk=pk)
        except Location.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = Location_Serializer(queryset)
        return Response(serializer.data, status=status.HTTP_302_FOUND)



# Generated by Django 4.2 on 2023-05-09 22:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='County',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('county', models.CharField(max_length=50)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'County/Municipality',
                'verbose_name_plural': 'County/Municipality',
            },
        ),
        migrations.CreateModel(
            name='Regions',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('region', models.CharField(max_length=20)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Region',
                'verbose_name_plural': 'Region',
            },
        ),
        migrations.CreateModel(
            name='SubCounty',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('sub_county', models.CharField(max_length=50)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('county', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='districts.county')),
            ],
            options={
                'verbose_name': 'Sub-County/Town Council/Division',
                'verbose_name_plural': 'Sub-County/Town Council/Division',
            },
        ),
        migrations.CreateModel(
            name='Parish',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('parish', models.CharField(max_length=50)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sub_county', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='districts.subcounty')),
            ],
            options={
                'verbose_name': 'Parish/Ward',
                'verbose_name_plural': 'Parish/Ward',
            },
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('latitude', models.FloatField()),
                ('longitude', models.FloatField()),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('parish', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='districts.parish')),
            ],
            options={
                'verbose_name': 'Location',
                'verbose_name_plural': 'Location',
            },
        ),
        migrations.CreateModel(
            name='Districts',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('district', models.CharField(max_length=50)),
                ('population', models.IntegerField()),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('region', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='districts.regions')),
            ],
            options={
                'verbose_name': 'District/City',
                'verbose_name_plural': 'District/City',
            },
        ),
        migrations.AddField(
            model_name='county',
            name='district',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='districts.districts'),
        ),
    ]



from django.contrib import admin

from districts.models import County, Regions, Districts, SubCounty, Parish, Location

# Regions Admin Model.
class Regions_Admin(admin.ModelAdmin):
    fields = [
        "region",
    ]

    list_display = (
        "id", "region", "created_at", "updated_at"
    )

admin.site.register(Regions, Regions_Admin)


# Districts Admin Model.
class Districts_Admin(admin.ModelAdmin):
    fields = [
        "region", "district",
    ]

    list_display = (
        "id", "region", "district", "created_at", "updated_at"
    )

admin.site.register(Districts, Districts_Admin)


# County Admin Model.
class County_Admin(admin.ModelAdmin):
    fields = [
        "district", "county",
    ]

    list_display = (
        "id", "district", "county", "created_at", "updated_at"
    )

admin.site.register(County, County_Admin)


# SubCounty Admin Model.
class SubCounty_Admin(admin.ModelAdmin):
    fields = [
        "county", "sub_county",
    ]

    list_display = (
        "id", "county", "sub_county", "created_at", "updated_at"
    )

admin.site.register(SubCounty, SubCounty_Admin)


# Parish Admin Model.
class Parish_Admin(admin.ModelAdmin):
    fields = [
        "sub_county", "parish",
    ]

    list_display = (
        "id", "sub_county", "parish", "created_at", "updated_at"
    )

admin.site.register(Parish, Parish_Admin)


# Location Admin Model.
class Location_Admin(admin.ModelAdmin):
    fields = [
        "parish", "latitude", "longitude",
    ]

    list_display = (
        "id", "parish", "latitude", "longitude", "created_at", "updated_at"
    )

admin.site.register(Location, Location_Admin)
